package at.ac.uibk.gitsearch.config.prometheus;

import io.prometheus.client.CollectorRegistry;
import java.util.Set;
import javax.annotation.Nullable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.actuate.endpoint.annotation.ReadOperation;
import org.springframework.boot.actuate.endpoint.web.WebEndpointResponse;
import org.springframework.boot.actuate.metrics.export.prometheus.PrometheusScrapeEndpoint;
import org.springframework.boot.actuate.metrics.export.prometheus.TextOutputFormat;

/**
 * Defines the "Skeleton" for a custom prometheus scrape endpoint.
 * To define a new scrape Endpoint extend this class and add @(Web)Endpoint(id = 'id')-Annotation.
 *
 *
 * @author Johannes Kainz
 */
public class CustomPrometheusScrapeEndpoint {

    @SuppressWarnings("unused")
    private final Logger log = LoggerFactory.getLogger(CustomPrometheusScrapeEndpoint.class);

    private final PrometheusScrapeEndpoint prometheusScrapeEndpoint;

    protected CustomPrometheusScrapeEndpoint(CollectorRegistry collectorRegistry) {
        this.prometheusScrapeEndpoint = new PrometheusScrapeEndpoint(collectorRegistry);
    }

    @ReadOperation(producesFrom = TextOutputFormat.class)
    public WebEndpointResponse<String> prometheusScrap(TextOutputFormat format, @Nullable Set<String> includedNames) {
        return this.prometheusScrapeEndpoint.scrape(format, includedNames);
    }
}
