import { ComponentFixture, TestBed, waitForAsync, fakeAsync, tick } from '@angular/core/testing';
import { GitsearchTestModule } from '../../test.module';
import { PagesComponent } from 'app/editorialPages/pages.component';
import { PagesService } from 'app/shared/service/pages-service';
import { ActivatedRoute, Router } from '@angular/router';
import { PagesMarkDownViewerComponent } from 'app/editorialPages/markDownViewer/markDownViewer.component';
import { MarkdownComponent, MarkdownModule, MarkdownService, MarkedOptions } from 'ngx-markdown';
import { RouterTestingModule } from '@angular/router/testing';
import { PagesResolve } from 'app/editorialPages/pages.route';
import { Location } from '@angular/common';
import { of } from 'rxjs';
import { markedOptionsFactory, MyRenderer } from 'app/editorialPages/editorialPages.module';

describe('Component Tests', () => {
  describe('Pages Component', () => {
    let comp: PagesComponent;
    let fixture: ComponentFixture<PagesComponent>;
    let compMD: PagesMarkDownViewerComponent;
    let fixtureMD: ComponentFixture<PagesMarkDownViewerComponent>;
    let compMarkDown: MarkdownComponent;
    let fixtureMarkDown: ComponentFixture<MarkdownComponent>;
    let pagesService: PagesService;
    let router: Router;
    let location: Location;

    beforeEach(
      waitForAsync(() => {
        const pagesTestRoutes = {
          path: 'pages',
          children: [
            {
              path: '**',
              // loadChildren: () => import('app/editorialPages/editorialPages.module').then(m => m.PagesModule),
              resolve: {
                pagePath: PagesResolve,
              },
              component: PagesComponent,
            },
          ],
        };
        TestBed.configureTestingModule({
          imports: [
            GitsearchTestModule,
            RouterTestingModule.withRoutes([pagesTestRoutes]),
            MarkdownModule.forRoot({
              markedOptions: {
                provide: MarkedOptions,
                useFactory: markedOptionsFactory,
              },
            }),
          ],
          declarations: [PagesComponent, PagesMarkDownViewerComponent],
          providers: [
            MarkdownService,
            {
              provide: ActivatedRoute,
              useValue: {
                url: of([]),
                data: of({ pagePath: 'start' }),
                params: of({ id: 123 }),
                queryParams: of([]),
                outlet: 'Hello World',
                component: '1',
                tagged: 'hello WORLD',
              },
            },
          ],
        })
          .overrideTemplate(PagesComponent, '')
          .overrideTemplate(
            PagesMarkDownViewerComponent,
            '<div *ngIf="page"><markdown #pageMarkDown ngPreserveWhitespaces [katexOptions]="katexOptions" [data]="page.content" (ready)="fixLinksToGoViaRouter()"></markdown></div>'
          )
          .compileComponents();
      })
    );

    beforeEach(() => {
      fixture = TestBed.createComponent(PagesComponent);

      /*
            const translateService = fixture.debugElement.injector.get(TranslateService);
            translateService.currentLang = "de";
            spyOn(translateService, 'onLangChange').and.returnValue(of('de'));
            */
      comp = fixture.componentInstance;
      fixtureMD = TestBed.createComponent(PagesMarkDownViewerComponent);
      compMD = fixtureMD.componentInstance;
      fixtureMarkDown = TestBed.createComponent(MarkdownComponent);
      compMarkDown = fixtureMarkDown.componentInstance;
      router = TestBed.get(Router);
      location = TestBed.get(Location);
      //            router = fixture.debugElement.injector.get(Router);
      router.initialNavigation();
      pagesService = fixture.debugElement.injector.get(PagesService);
    });

    it('check routing of pages component', fakeAsync(() => {
      /** unfortunatelly very rudimentary :-(
       * router.navigate does not trigger page observers
       */
      // GIVEN

      comp.ngOnInit();
      compMD.ngOnInit();

      router.navigate(['pages/en/start']);

      spyOn(pagesService, 'getPage').and.callFake((pagePath: string) => {
        return of({ path: pagePath, content: '#head \n hello world' });
      });

      tick();
      // WHEN,
      compMD.path = '/start';

      fixtureMD.detectChanges();

      // THEN
      expect(location.path()).toBe('/pages/en/start');
      expect(comp.pagePath).toBe('start');
      expect(compMD.page.path).toBe('en/start');
    }));

    it('check markdown generation', fakeAsync(() => {
      // GIVEN

      spyOn(pagesService, 'getPage').and.callFake((pagePath: string) => {
        return of({
          path: pagePath,
          content:
            '# head\n[Wie stelle ich Inhalte zur Verfügung](de/publishers/howto)\n![image](uploads/78141952f6701c38c37b0bd2d8e01658/image.png)',
        });
      });

      const myRenderer = compMarkDown.markdownService.renderer;
      spyOn(myRenderer as MyRenderer, 'getAttachmentURL').and.returnValue(of('/fake/attachment'));
      compMD.ngOnInit();

      tick();
      // WHEN,
      compMD.path = '/start';
      fixtureMD.detectChanges();

      compMarkDown.data = compMD.page.content;
      // const renderedResult = compMarkDown.render(compMarkDown.data);
      fixtureMarkDown.detectChanges();

      // const nativeElem = fixtureMarkDown.debugElement.nativeElement;

      // const innerHtml = nativeElem.innerHtml;

      // THEN
    }));
  });
});
